/* z81/xz81, Linux console and X ZX81/ZX80 emulators.
 * Copyright (C) 1994 Ian Collier. z81 changes (C) 1995-2001 Russell Marks.
 * Allegro version Copyright (C) 2015 Stefano Bodrato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <sys/time.h>
#include <allegro5/allegro.h>
//#include <allegro5/allegro_image.h>
#include <allegro5/allegro_primitives.h>
#include "common.h"
#include "sound.h"
#include "z80.h"
#include "allmain.h"


#ifdef ALLEGRO

extern ALLEGRO_EVENT_QUEUE *event_queue;
//extern ALLEGRO_TIMER *timer;

#endif


ALLEGRO_DISPLAY *display = NULL;
ALLEGRO_BITMAP *bitmap;
ALLEGRO_KEYBOARD_STATE keymap;
ALLEGRO_COLOR black;
ALLEGRO_COLOR white;

int hsize=ZX_VID_VGA_WIDTH,vsize=ZX_VID_VGA_HEIGHT;


void exit_program(void)
{
#if defined(ALLEGRO) || defined(OSS_SOUND_SUPPORT)
sound_end();
#endif
zxpclose();

#ifdef ALLEGRO
//al_destroy_timer(timer);
al_destroy_event_queue(event_queue);
#endif

al_destroy_bitmap(bitmap);
al_destroy_display(display);
al_uninstall_keyboard();
al_shutdown_primitives_addon();
//al_shutdown_image_addon();


exit(0);
}


/* redraw the screen */
void update_scrn(void)
{

int x,y,a,mask;
unsigned char *ptr,*optr,d;

  al_set_target_bitmap(bitmap);
  al_lock_bitmap(bitmap, ALLEGRO_PIXEL_FORMAT_ANY, ALLEGRO_LOCK_WRITEONLY);


for(y=0;y<vsize;y++)
  {
  ptr=scrnbmp+(y+ZX_VID_VGA_YOFS)*ZX_VID_FULLWIDTH/8+ZX_VID_VGA_XOFS/8;
  optr=scrnbmp_old+(ptr-scrnbmp);
  
  for(x=0;x<hsize;x+=8,ptr++,optr++)
    {
    d=*ptr;
    if(d!=*optr || refresh_screen) {
		  if(invert_screen)
			d=~d;
		  for(a=0,mask=128;a<8;a++,mask>>=1)
			al_put_pixel(x+a, y, (d&mask)?black:white);
	  }
    }
  }
  
  al_unlock_bitmap(bitmap);
  al_set_target_backbuffer(display);
  //al_draw_bitmap(bitmap,0,0,0);
  al_draw_scaled_bitmap(bitmap,0,0,hsize,vsize,0,0,hsize*SCALE,vsize*SCALE,0);

  al_flip_display();
/* now, copy new to old for next time */
memcpy(scrnbmp_old,scrnbmp,ZX_VID_FULLHEIGHT*ZX_VID_FULLWIDTH/8);

refresh_screen=0;
}

/* read keyboard and update keyports[] */
void check_events()
{
int b,y;
al_get_keyboard_state(&keymap);

al_get_keyboard_state(&keymap);

if(al_key_down(&keymap,ALLEGRO_KEY_ESCAPE) && !ignore_esc)
  reset81();

if(al_key_down(&keymap,ALLEGRO_KEY_F10))
  {
  while(al_key_down(&keymap,ALLEGRO_KEY_F10)) usleep(20000),al_get_keyboard_state(&keymap);
  exit_program();	/* F10 = quit */
  }

if(al_key_down(&keymap,ALLEGRO_KEY_F1))
  {
  help=!help;
  while(al_key_down(&keymap,ALLEGRO_KEY_F1)) usleep(20000),al_get_keyboard_state(&keymap);

  }

/* ugly, but there's no pleasant way to do this */

/* XXX ideally want to support DEL etc. */

for(y=0;y<8;y++)		/* 8 half-rows */
  {
  b=0;	/* we set bits in b as appropriate */
  switch(y)	/* below comments given in order b1->b5 */
    {
    /* left-hand side */
    case 0:	/* sft,z,x,c,v */
      if(al_key_down(&keymap, ALLEGRO_KEY_LSHIFT) ||
         al_key_down(&keymap, ALLEGRO_KEY_RSHIFT) ||
         al_key_down(&keymap, ALLEGRO_KEY_LCTRL) ||
         al_key_down(&keymap, ALLEGRO_KEY_RCTRL))
        b|=1;
      if(al_key_down(&keymap, ALLEGRO_KEY_Z)) b|=2;
      if(al_key_down(&keymap, ALLEGRO_KEY_X)) b|=4;
      if(al_key_down(&keymap, ALLEGRO_KEY_C)) b|=8;
      if(al_key_down(&keymap, ALLEGRO_KEY_V)) b|=16;
      break;
    case 1:	/* a,s,d,f,g */
      if(al_key_down(&keymap, ALLEGRO_KEY_A)) b|=1;
      if(al_key_down(&keymap, ALLEGRO_KEY_S)) b|=2;
      if(al_key_down(&keymap, ALLEGRO_KEY_D)) b|=4;
      if(al_key_down(&keymap, ALLEGRO_KEY_F)) b|=8;
      if(al_key_down(&keymap, ALLEGRO_KEY_G)) b|=16;
      break;
    case 2:	/* q,w,e,r,t */
      if(al_key_down(&keymap, ALLEGRO_KEY_Q)) b|=1;
      if(al_key_down(&keymap, ALLEGRO_KEY_W)) b|=2;
      if(al_key_down(&keymap, ALLEGRO_KEY_E)) b|=4;
      if(al_key_down(&keymap, ALLEGRO_KEY_R)) b|=8;
      if(al_key_down(&keymap, ALLEGRO_KEY_T)) b|=16;
      break;
    case 3:	/* 1,2,3,4,5 */
      if(al_key_down(&keymap, ALLEGRO_KEY_1)) b|=1;
      if(al_key_down(&keymap, ALLEGRO_KEY_2)) b|=2;
      if(al_key_down(&keymap, ALLEGRO_KEY_3)) b|=4;
      if(al_key_down(&keymap, ALLEGRO_KEY_4)) b|=8;
      if(al_key_down(&keymap, ALLEGRO_KEY_5)) b|=16;
      break;

    /* right-hand side */
    case 4:	/* 0,9,8,7,6 */
      if(al_key_down(&keymap, ALLEGRO_KEY_0)) b|=1;
      if(al_key_down(&keymap, ALLEGRO_KEY_9)) b|=2;
      if(al_key_down(&keymap, ALLEGRO_KEY_8)) b|=4;
      if(al_key_down(&keymap, ALLEGRO_KEY_7)) b|=8;
      if(al_key_down(&keymap, ALLEGRO_KEY_6)) b|=16;
      break;
    case 5:	/* p,o,i,u,y */
      if(al_key_down(&keymap, ALLEGRO_KEY_P)) b|=1;
      if(al_key_down(&keymap, ALLEGRO_KEY_O)) b|=2;
      if(al_key_down(&keymap, ALLEGRO_KEY_I)) b|=4;
      if(al_key_down(&keymap, ALLEGRO_KEY_U)) b|=8;
      if(al_key_down(&keymap, ALLEGRO_KEY_Y)) b|=16;
      break;
    case 6:	/* ent,l,k,j,h */
      if(al_key_down(&keymap, ALLEGRO_KEY_ENTER)) b|=1;
      if(al_key_down(&keymap, ALLEGRO_KEY_L)) b|=2;
      if(al_key_down(&keymap, ALLEGRO_KEY_K)) b|=4;
      if(al_key_down(&keymap, ALLEGRO_KEY_J)) b|=8;
      if(al_key_down(&keymap, ALLEGRO_KEY_H)) b|=16;
      break;
    case 7:	/* spc,dot,m,n,b */
      if(al_key_down(&keymap, ALLEGRO_KEY_SPACE)) b|=1;
      if(al_key_down(&keymap, ALLEGRO_KEY_FULLSTOP)) b|=2;
      if(al_key_down(&keymap, ALLEGRO_KEY_M)) b|=4;
      if(al_key_down(&keymap, ALLEGRO_KEY_N)) b|=8;
      if(al_key_down(&keymap, ALLEGRO_KEY_B)) b|=16;
      break;
    }
  
  keyports[y]=((b^31)|0xe0);	/* some things need top bits to be 1 */
  }
}



int main(int argc,char *argv[])
{

black=al_map_rgb_f(0.05, 0.05, 0.05);
white=al_map_rgb_f(0.94, 0.94, 0.94);

// if (!al_init() || !al_init_primitives_addon() || !al_init_image_addon()) {
if (!al_init() || !al_init_primitives_addon() ) {
   return -1;
}
al_set_new_display_flags(ALLEGRO_WINDOWED | ALLEGRO_RESIZABLE);

parseoptions(argc,argv);

initmem();
loadhelp();
zxpopen();

display = al_create_display(hsize*SCALE, vsize*SCALE);
if (!display) {
   return -1;
}
al_set_new_bitmap_flags(ALLEGRO_VIDEO_BITMAP|ALLEGRO_NO_PRESERVE_TEXTURE);
bitmap = al_create_bitmap(hsize*SCALE, vsize*SCALE);
al_set_window_title(display, "az81");
al_set_new_display_flags(ALLEGRO_WINDOWED | ALLEGRO_RESIZABLE);


refresh_screen=1;

if(!al_install_keyboard()) {
   return -1;
}

startsigsandtimer();

#if defined(ALLEGRO) || defined(OSS_SOUND_SUPPORT)
if(sound)
  sound_init();
#endif

mainloop();

/* not reached, but just in case */
exit_program();
return(0);	/* make -Wall happy */
}
